declare("ledger", {})
ledger.version = '0.4.4'
--[url=http://bespin.org/~draugath/vo/ledger/ledger_0.4.2.zip]ledger_0.4.2.zip[/url]

-- TODO
-- rework categories and filters
-- 



-- Functions
ledger.init = {}
ledger.log = {}
ledger.record = {}

dofile('vars.lua')
dofile('ui.lua')
dofile('fuzzytime.lua')

function ledger.save(tbl)
	if (type(tbl) ~= 'table') then return nil, 'Table expected, got '..type(tbl)..'.' end
	return pcall(SaveSystemNotes, spickle(tbl), ledger.notesid)
end


function ledger.log(event, data)
	local credits, target, category, typeof, notes = 0
	local rfilter = ledger.filter.record


	if (event == "CHAT_MSG_CONFIRMATION") then
		-- Event: Purchase failed.
		if     (data.msg:match('Items purchase failed%.')) then
			ledger.vars.purchase_failed = true

		-- Event: /givemoney - Send
		elseif (rfilter.player.send and data.msg:match("Initiating transfer")) then 
			-- Event:CHAT_MSG_CONFIRMATION - Data:msg="Initiating transfer of 1 credits to Conflict Diamond..."
			credits, target = data.msg:match("([%d,]+) credits to (.*)%.%.%.")
			category, typeof = 'player.send', 'Debit'

		-- Event: Reload ship
		elseif (rfilter.ship.reload and data.msg:match("^All ammo") and not data.msg:match("^Items purchased%.$")) then
			-- Event:CHAT_MSG_CONFIRMATION - Data:msg="1x of Purified Water purchased for a total price of 8c."
			target, credits = data.msg:match("^(.*) purchased.-([%d,]+)c%.$")
			category, typeof = 'ship.reload', 'Debit'

		-- Event: Purchase ship
		elseif (rfilter.ship.buy and ledger.vars.shiplist[data.msg:match("of (.*) purchased")] and not data.msg:match("^Items purchased%.$")) then
			-- Event:CHAT_MSG_CONFIRMATION - Data:msg="1x of EC-89 purchased for a total price of 8c."
			target, credits = data.msg:match("^(.*) purchased.-([%d,]+)c%.$")
			category, typeof = 'ship.buy', 'Debit'
		
		-- Event: Purchase item
		elseif (rfilter.station.buy and data.msg:match("purchased") and not data.msg:match("^Items purchased%.$")) then
			-- Event:CHAT_MSG_CONFIRMATION - Data:msg="1x of Purified Water purchased for a total price of 8c."
			target, credits = data.msg:match("^(.*) purchased.-([%d,]+)c%.$")
			category, typeof = 'station.buy', 'Debit'

		-- Event: Sell ship
		elseif (rfilter.ship.sell and ledger.vars.shiplist[data.msg:match("of (.*) sold")]) then
			-- Event:CHAT_MSG_CONFIRMATION - Data:msg="1x of EC-89 sold for a total amount of 0c (loss of 0c)"
			target, credits, notes = data.msg:match("^(.*) sold.-([%d,]+)c (%(.*%))$")
			category, typeof = 'ship.sell', 'Credit'

		-- Event: Sell item
		elseif (rfilter.station.sell and data.msg:match("sold")) then
			-- Event:CHAT_MSG_CONFIRMATION - Data:msg="1x of Purified Water sold for a total amount of 7c (loss of 1c)"
			target, credits, notes = data.msg:match("^(.*) sold.-([%d,]+)c (%(.*%))$")
			category, typeof = 'station.sell', 'Credit'

		-- Event: Repair ship
		elseif (rfilter.ship.repair and data.msg:match("^Ship repaired")) then
			-- Event:CHAT_MSG_CONFIRMATION - Data:msg="Ship repaired for a total price of 59c."
			target, credits = data.msg:match("^(Ship repair).-([%d,]+)c%.")
			category, typeof = 'ship.repair', 'Debit'

		-- Event: Rebuy last ship
		elseif (rfilter.station.buy and data.msg:match("^Total cost of purchase")) then
			-- Event:CHAT_MSG_CONFIRMATION - Data:msg="Total cost of purchase is 577 credits"
			target, credits = "Last ship purchase", data.msg:match("([%d,]+) credits")
			category, typeof = 'station.buy', 'Debit'
		end
	end

	if (event == "CHAT_MSG_SECTORD") then
		-- Event: /givemoney - Receive
		if (rfilter.player.receive and data.msg:match("sent you %d+ credits%.")) then
			-- Event:CHAT_MSG_SECTORD - Data:msg="Conflict Diamond sent you 1 credits."
			target, credits = data.msg:match("^(.*) sent you ([%d,]+) credits%.$")
			category, typeof = 'player.receive', 'Credit'
		end
	end

	if (event == "CHAT_MSG_SECTORD_MISSION") then
	end

	if (event == 'CHAT_MSG_SERVER') then
		-- Event: Deposit to guild bank
		if     (rfilter.guild.deposit and data.msg:match("deposited into the guild bank")) then
			-- Event: CHAT_MSG_SERVER - Data:msg="10000 c has been deposited into the guild bank."
			target, credits = 'Deposit into guild bank', data.msg:match('^(%d+) c')
			category, typeof = 'guild.deposit', 'Debit'

		-- Event: Withdraw from guild bank.
		elseif (rfilter.guild.withdraw and data.msg:match("from the guild bank")) then
			-- Event: CHAT_MSG_SERVER - Data:msg="10000 c has been added to your account from the guild bank."
			target, credits = 'Withdrawl from guild bank', data.msg:match('^(%d+) c')
			category, typeof = 'guild.withdraw', 'Credit'
		end
	end

	
	if (target and credits) then ledger.record(category, target, typeof, credits, notes) end
end

function ledger.record(category, target, typeof, amount, notes)
	local str = notes or ''
	if (ledger.vars.purchase_failed) then 
		str = 'Items purchase failed. Not all items were purchased because not all items are available at this station.'
		ledger.vars.purchase_failed = false
	end
	
	amount = tonumber((string.gsub(amount, ',', '')))
	typeof = string.upper(typeof)
	if (typeof == 'DEBIT' and amount > 0) then amount = -amount end
	
	table.insert(ledger.list, {category = category, time = os.time(), reason = target, type = typeof, amount = tonumber(amount), notes = str})
	ledger.save(ledger.list)
end

function ledger.init:OnEvent(event, data)
	UnregisterEvent(self, "PLAYER_ENTERED_GAME")
	ledger.list = unspickle(LoadSystemNotes(ledger.notesid)) or {}
	ledger.viewer.build_dialog()
	ledger.viewer.edit.build_dialog()
	--ledger.viewer.options.build_dialog()
	print(ledger.vars.fontcolor..'Ledger Loaded  [ v'..ledger.version..' ]')
end

RegisterEvent(ledger.init, 'PLAYER_ENTERED_GAME')
RegisterEvent(ledger.log,  'CHAT_MSG_CONFIRMATION')
RegisterEvent(ledger.log,  'CHAT_MSG_SECTORD')
RegisterEvent(ledger.log,  'CHAT_MSG_SECTORD_MISSION')
RegisterEvent(ledger.log,  'CHAT_MSG_SERVER')

RegisterUserCommand("ledger", function(_,args) 
	local fontcolor = ledger.vars.fontcolor

	if (args == nil) then
		ledger.viewer.ui.dialog:Show()

	elseif (args[1] == 'fix042') then
		local idxs = {}
		for i = 1, #ledger.list do
			if (ledger.list[i].category == '') then table.insert(idxs, i) end
		end
		for i = #idxs, 1, -1 do
			table.remove(ledger.list, idxs[i]) 
		end
		ledger.save(ledger.list)
		print(fontcolor..'Blank records have been deleted.')
		
	elseif (args[1] == 'reset') then 
		ledger.list = {}
		ledger.save(ledger.list)
		ledger.vars.startcredits = GetMoney()
		gkini.WriteInt('Ledger', 'startcredits', ledger.vars.startcredits)
		print(fontcolor..'Ledger histories have been reset.')

	elseif (args[1] == 'lastten') then
		print("Placeholder: for printing last ten transactions")
	end
end)




-- *You complete a procurement mission
-- Event:CHAT_MSG_SECTORD_MISSION - Data:missionid=1587,msg="66558 credits received (41958 reimbursal)"
-- Event:MISSION_NOTIFICATION - Data:1587
-- Event:CHAT_MSG_SECTORD_MISSION - Data:missionid=1587,msg="Great!  Thanks for the delivery.  Feel free to sign up again if you're looking for more work."
-- Event:CHAT_MSG_SECTORD_MISSION - Data:missionid=1587,msg="Mission completed in 00:28.  Time bonus received, 2200 credits.<button title=\"Contact\"/>"
-- [Sat Nov 27 02:27:21 2010] 68758 credits added to your account.
-- Event:CHAT_MSG_SECTORD_MISSION - Data:missionid=0,msg="68758 credits added to your account."

-- *Partial procurement delivery
-- Event:CHAT_MSG_SECTORD_MISSION - Data:missionid=1593,msg="1 item delivered.  7 items remaining.  Go get the rest from Nyrius F-6, Asuin Tech.  (4662 credits received)<button title=\"Contact\"/>"
-- Event:CHAT_MSG_SECTORD_MISSION - Data:missionid=1595,msg="5 items delivered.  8 items remaining.  Go get the rest from Nyrius F-6, Asuin Tech.  (23310 credits received)<button title=\"Contact\"/>"
