ledger.viewer                 = {}
ledger.viewer.ui              = {}
ledger.viewer.ui.dialog       = {}

ledger.viewer.build_dialog    = {}
ledger.viewer.filter          = {}
ledger.viewer.matrix          = {}

ledger.viewer.sort_col        = 'time'
ledger.viewer.sort_dir        = 'desc'
ledger.viewer.updated         = false
ledger.viewer.matrix_list_map = {} --holds an index map pointing from the matrix line to the list index

local state_map = {ON = true,  OFF = false}
local trans_map = {ON = 'YES', OFF = 'NO'}

local function capitalize(str) return (str:gsub('^.', string.upper, 1)) end

local function Alarm(t, m, b1) -- creates a popup window with a message and a single button
	if (not (t and m and b1)) then return nil end
	t, m, b1 = tostring(t), tostring(m), tostring(b1)
	local function create_button(t) return iup.stationbutton{title = t, action = function(self) iup.GetDialog(self):hide() end} end
	b1 = create_button(b1)
	local button_box = iup.hbox{iup.fill{}, b1, iup.fill{}}
	local dialog = iup.dialog{iup.vbox{iup.fill{}, iup.hbox{iup.fill{}, iup.pdarootframe{iup.vbox{iup.hbox{iup.label{title = m, expand = 'HORIZONTAL'}}, button_box,}},iup.fill{},}, iup.fill{},}, title = t, defaultenter = b1, defaultesc = b1, border = "NO", resize = "NO", menubox = "YES", topmost = "YES"}
	dialog:map()
	iup.Popup(dialog, iup.CENTER, iup.CENTER)
	return true
end

local function check_date_part(ihandle, part)
	local val_limits = {day = {min = 1, max = 31}, month = {min = 1, max = 12}, year = {min = 1970, max = 4000}}
	local val = tonumber(ihandle.value) or 0
	local ret = (val >= val_limits[part].min and val <= val_limits[part].max)
	if (not ret) then Alarm('Invalid entry', capitalize(part)..' needs to be a value from '..val_limits[part].min..' - '..val_limits[part].max, 'OK') end
	return ret
end

local function toggle_state(tbl, property, state)
	if (type(tbl) ~= 'table' and type(tbl) ~= 'userdata') then return nil end

	if (type(tbl) == 'userdata') then
		if (property == 'value' and not iup.GetType(tbl) == 'toggle') then return end
		tbl[property] = state

	elseif (type(tbl) == 'table') then
		for _, v in pairs(tbl) do
			toggle_state(v, property, state)
		end
	end
	return true
end

local function toggle_active(tbl, yesno) 
	if (yesno ~= 'YES' and yesno ~= 'NO') then return nil end
	return toggle_state(tbl, 'active', yesno)
end

local function toggle_value(tbl, onoff)
	if (onoff ~= 'ON' and onoff ~= 'OFF') then return nil end
	return toggle_state(tbl, 'value', onoff)
end

local function toggle_visible(tbl, yesno) 
	if (yesno ~= 'YES' and yesno ~= 'NO') then return nil end
	return toggle_state(tbl, 'visible', yesno)
end


local function format_amount(amount)
	amount = tonumber(amount)
	if (amount < 0) then 
		return '('..comma_value(-amount)..')' 
	else 
		return comma_value(amount) 
	end
end

local tinsert = table.insert
local tsort = table.sort
local function xor(a, b) return ((a and not b) or (not a and b)) end

local function label(title)
	return iup.label{title = title} 
end

local function fill(expand, size)
	local ctl = iup.fill{}
	if (expand) then ctl.expand = expand end
	if (size) then ctl.size = size end
	return ctl
end

local function text(size, active)
	local ctl = iup.text{}
	if (size) then ctl.size = size end
	if (active) then ctl.active = active end
	return ctl
end

local function list(tbl, size, active, dropdown)
	local ctl = iup.stationsubsublist{tbl; size = size, active = active, dropdown = dropdown, visible_items = 10}
	for i, v in ipairs(tbl) do ctl[i] = v end
	return ctl
end

local function sortlist(list, sort_col, sort_dir)
	if (list == nil) then return nil, "list is 'nil'. Expected a 'table'" end
	if (sort_col == nil) then return nil, "sort_col is 'nil'.  Expected a 'string'" end
	if (sort_dir ~= nil and not xor(sort_dir ~= 'asc', sort_dir ~= 'desc')) then return nil, "invalid sort direction '"..tostring(sort_dir).."'" end
	
	if (sort_dir == nil) then
		-- If sort direction not provided, then set direction based on other criteria
		if (sort_col ~= ledger.viewer.sort_col) then 
			-- If the new sort column doesn't match the old, then default to ascending sort
			sort_dir = 'asc'
		else
			-- Switch sort direction to opposite of previous
			if (ledger.viewer.sort_dir == 'asc') then sort_dir = 'desc' else sort_dir = 'asc' end
		end
	else
		-- Otherwise use the sort direction provided
	end

	tsort(list, function(a,b) 
		if (sort_dir == 'desc') then a, b = b, a end
		if (type(list[1][sort_col]) == 'number') then
			return a[sort_col] < b[sort_col] 
		else
			return gkmisc.strnatcasecmp(a[sort_col],b[sort_col]) < 0 
		end
	end)

	ledger.viewer.sort_col = sort_col
	ledger.viewer.sort_dir = sort_dir
	return true
end

----------------------------------------------------

function ledger.viewer.matrix:New()
	local header = ledger.vars.header
	local header_reverse = ledger.vars.header_reverse
	local bg = ledger.vars.bg

	-- Instantiate
	local mat = iup.pdasubsubmatrix{numcol = 5, numcol_visible = 5, numlin = 0, numlin_visible = 25, hidefocus = 'YES', size='TWOTHIRD'}

	-- Initialize -- matrix headers, column alignments
	for _, v in pairs(header) do
		mat:setcell(0, v[1], v[2])
		mat["ALIGNMENT"..v[1]] = v[3]
	end

	function mat:setbg(lin, color)
		if (color) then self:setattribute("BGCOLOR", lin, -1, color) else self:setattribute("BGCOLOR", lin, -1, bg[lin % 2]) end
	end

	function mat:setfg(color, lin, col)
		if (col) then self['FGCOLOR'..lin..':'..col] = color else self:setattribute("FGCOLOR", lin, -1, bg[lin % 2]) end
	end

	function mat:setval(lin, col, val) 	self:setcell(lin, col, ' '..val..' ') end

	function mat:update()
		local list = ledger.list
		local count, total = 1, ledger.vars.startcredits
		local vfilter = ledger.filter.view
		local rlookup = ledger.filter.view.reverse_lookup
		local search = ledger.viewer.ui.searchbox.value:lower()

		local function checkfilter(major, minor, t)
			if (not rlookup[major]) then return false end
			if (not rlookup[major][minor]) then return false end

			local master = vfilter[major].master
			local filt = vfilter[major]
			local idx = rlookup[major][minor]

			if (major == 'date') then 
				if (not t) then return nil end
				if (state_map[master.value]) then return filt.cmp:check(t) end
				return true
			end
			return state_map[master.value] or state_map[filt[idx].value]
		end

		local function setline(lin, tbl)
			for i = 1, #tbl do
				
				self:setval(lin, i, tbl[i])
				if (header_reverse[i] == 'amount' and tbl[i]:match('^%(')) then 
					self:setfg(lin, i, '255 50 50')
				else
					self:setfg(lin, i, '255 255 255')
				end
			end
		end

		local function escape_pattern(str)
			if (type(str) ~= 'string') then return nil, 'Expected string, got '..type(str)..'.' end
			return str:gsub('(%p)', '%%%1')
		end

		local function find_match(tbl, key)
			return tbl[header[key][1]]:lower():match(escape_pattern(search))
		end

		ledger.viewer.matrix_list_map = {}
		self.dellin = "1--1"
		-- Populate the matrix
		for i, v in ipairs(list) do
			local major, minor = v.category:match("(.-)%.(.*)")
			if (checkfilter(major, minor) and checkfilter('type', v.type:lower()) and checkfilter('date', 'check', v.time)) then
				self.numlin = count
				local lin_data = {}
				for k, w in pairs(v) do
					-- test.print('k='..tostring(k))
					if (k ~= 'category') then
						if (k == 'time') then 
							w = fuzzy_ledger.diff(os.time(), w, ledger.format.fuzziness)
							if (type(w) == 'number') then w = os.date(ledger.format.date, w) end
						elseif (k == 'amount') then
							total = total + w
							w = format_amount(w)
						end
						lin_data[header[k][1]] = w
					end
				end
				-- test.print(spickle(lin_data))
				-- Add the line to the matrix if searchbox is blank or the search pattern is found in reason.
				if (search == '' or find_match(lin_data, 'reason') or find_match(lin_data, 'notes')) then 
					setline(count, lin_data)
					self:setbg(count, bg[count % 2])
					ledger.viewer.matrix_list_map[count] =  i
					count = count + 1
				end
 			end
		end
		ledger.viewer.ui.total.title = format_amount(total)
		if (ledger.viewer.ui.total.title:match('^%(')) then 
			ledger.viewer.ui.total.fgcolor = '255 50 50'
		else
			ledger.viewer.ui.total.fgcolor = '255 255 255'
		end
	end

	function mat:action(c, lin, col, active, after) 
		--test.print('c='..c..' / lin='..lin..' / col='..col..'/ active='..active..' / after='..after)
	end

	function mat:click_cb(lin, col)
		local sort_col = header_reverse[col]
		if (lin == 0) then
			sortlist(ledger.list, sort_col)
			self:update()
		else
			if (col ~= header.notes[1]) then return iup.IGNORE end
		end
	end

	function mat:enteritem_cb(lin, col)
		self:setbg(lin, bg[2])
		return iup.DEFAULT
	end

	function mat:leaveitem_cb(lin, col)
		self:setbg(lin, bg[lin % 2])
		return iup.DEFAULT
	end

	function mat:edition_cb(line, col, mode)
		-- test.print('line='..line..' / col='..col..' / mode='..mode)
		local list = ledger.list
		local map = ledger.viewer.matrix_list_map
		local edit = ledger.viewer.edit.ui
		if (mode == 1) then
			edit.dialog.curline = line
			edit.date:set(list[map[line]].date)
			edit.reason:set(list[map[line]].reason)
			edit.amount:set(format_amount(list[map[line]].amount))
			edit.textbox.value = list[map[line]].notes
			edit.dialog:popup(iup.CENTER, iup.CENTER)
			iup.Refresh(edit.dialog)
		end
		return iup.IGNORE
	end
	return mat
end


function ledger.viewer.filter:New(listname)
	local vfilter = ledger.filter.view[listname]
	local items = vfilter.list
	local capname = capitalize(listname)
	local vbox = {}
	local subfilters = iup.vbox{}

	local function create_toggle(title, value, func)
		if (not title) then return nil, "Expected a string for title." end
		value = value or 'ON'
		func = func or function(self)
			local master = vfilter.master
			local onoff = 'ON'
			for _, v in ipairs(vfilter) do
				if (not state_map[v.value]) then onoff = 'OFF'; break end
			end
			if (xor(state_map[master.value], state_map[onoff])) then toggle_value(master, onoff) end
		end

		return iup.stationtoggle{title = title, value = value, action = func}
	end

	if (vfilter.ctl_type == 'toggle') then
		vfilter.master = create_toggle(capname, nil, function(self)
			local state = self.value
			toggle_value(vfilter, state)
		end)

		for i, v in ipairs(items) do
			local tog = create_toggle(v)
			vfilter[i] = tog
			iup.Append(subfilters, tog)
		end

		vbox = iup.vbox{
			iup.hbox{vfilter.master, fill()},
			iup.hbox{fill(nil, 10),	subfilters,},
			fill(nil, 10),
		}

	elseif (vfilter.ctl_type == 'list') then
		vfilter.master = create_toggle(capname, 'OFF', function(self)
			local m_active = trans_map[vfilter.master.value]
			toggle_active({vfilter.cmp, vfilter.zbox}, m_active)
		end)

		vfilter.cmp = iup.stationsublist{dropdown = 'YES', active = 'NO'}

		vfilter.cmp.action = function(self, t, i, v)
			if (i == 1) then vfilter.zbox.value = vfilter.date_predef
			elseif (i == 2 or i == 3) then
				vfilter.zbox.value = vfilter.date_custom
				toggle_visible(vfilter.date2_box, 'YES')
			elseif (i == 4 or i == 5) then
				vfilter.zbox.value = vfilter.date_custom
				toggle_visible(vfilter.date2_box, 'NO')
			end
		end

		vfilter.cmp.check = function(self, t)
			local i = tonumber(self.value)
			local date1 = vfilter.date1
			date1 = os.time({year = date1.year[date1.year.value], month = date1.month.value, day = date1.day.value})
			local date2 = vfilter.date2
			date2 = os.time({year = date2.year[date2.year.value], month = date2.month.value, day = date2.day.value, hour = 23, min = 59, sec = 59})
	
			local day1 = 86399
			local day7 = 604799
			local day30 = 2591999
			if (i == 1) then
				if (not vfilter.date_predef.value) then return nil end
				local title = vfilter.date_predef.value.title
				date2 = os.date('*t'); date2.hour = 23; date2.min = 59; date2.sec = 59; date2.wday = nil; date2.yday = nil; date2.isdst = nil
				date2 = os.time(date2)
				if (title == 'Today') then date1 = date2 - day1; i = 2
				elseif (title == 'Last 7 days') then date1 = date2 - day7; i = 2
				elseif (title == 'Last 30 days') then date1 = date2 - day30; i = 2
				end
			end

			if     (i == 2) then   if (t >= date1 and t <= date2)   then return true end
			elseif (i == 3) then   if (t < date1 or t > date2)      then return true end
			elseif (i == 4) then   if (t >= date1)                  then return true end
			elseif (i == 5) then   if (t < date1)                   then return true end
			end
			return false
		end

		for i, v in ipairs(vfilter.list) do vfilter.cmp[i] = v end

		local days = {}; for i = 1, 31 do days[i] = i end
		local months = ledger.vars.months
		local years = {}; for i = os.date('%Y'), 2010, -1 do table.insert(years, i) end

		vfilter.date1 = {day = list(days, 35, 'YES', 'YES'), month = list(months, 40, 'YES', 'YES'), year = list(years, 50, 'YES', 'YES')}
		local date1 = vfilter.date1
		date1.day.value = os.date('%d'); date1.month.value = os.date('%m'); date1.year.value = 1
		local date1_hbox = iup.hbox{date1.day, label(' / '), date1.month, label(' / '), date1.year}

		vfilter.date2 = {day = list(days, 35, 'YES', 'YES'), month = list(months, 40, 'YES', 'YES'), year = list(years, 50, 'YES', 'YES')}
		local date2 = vfilter.date2
		date2.day.value = os.date('%d'); date2.month.value = os.date('%m'); date2.year.value = 1
		vfilter.date2_box = iup.vbox{iup.label{title = 'and', expand = 'HORIZONTAL', alignment = 'ACENTER'}, iup.hbox{date2.day, label(' / '), date2.month, label(' / '), date2.year}}
		
		vfilter.date_custom = iup.vbox{ date1_hbox, vfilter.date2_box}

		vfilter.date_predef = iup.radio{
			iup.vbox{
				iup.stationtoggle{title = 'Today'},
				iup.stationtoggle{title = 'Last 7 days'},
				iup.stationtoggle{title = 'Last 30 days'}
			}
		}

		vfilter.zbox = iup.zbox{vfilter.date_predef, vfilter.date_custom, alignment = 'NW', active = 'NO'}

		vbox = iup.vbox{
			iup.vbox{vfilter.master, fill(nil, 5)},
			iup.vbox{
				iup.hbox{fill(nil, 10), vfilter.cmp,},
				fill(nil, 5),
				iup.hbox{
					fill(nil, 10),
					vfilter.zbox,
				},
			},
			fill(nil, 10),
		}
	end
	return vbox
end


ledger.viewer.ui.matrix         = ledger.viewer.matrix:New()
ledger.viewer.ui.filter_button  = iup.stationbutton{title = 'Filter', action = function(self)
		ledger.viewer.ui.matrix:update()
	end}
ledger.viewer.ui.reset          = iup.stationbutton{title = 'Reset', action = function(self)
		local vfilter = ledger.filter.view
		local k, control

		for k in pairs(vfilter) do
			if (k ~= 'reverse_lookup') then
				if (k == 'date') then vfilter[k].master.value = 'OFF' else vfilter[k].master.value = 'ON' end
				vfilter[k].master:action()
			end
		end
		ledger.viewer.ui.matrix:update()
	end}
ledger.viewer.ui.total          = iup.label{title = ''}
ledger.viewer.ui.searchbox      = iup.text{value = '', expand = 'HORIZONTAL'}
ledger.viewer.ui.search         = iup.stationbutton{title = 'Search', action = function(self)
		ledger.viewer.ui.matrix:update()
	end}
ledger.viewer.ui.clear          = iup.stationbutton{title = 'Clear', action = function(self)
		ledger.viewer.ui.searchbox.value = ''
		ledger.viewer.ui.matrix:update()
	end}

ledger.viewer.ui.options        = iup.stationbutton{title = 'Options', action = function(self)
		local options = ledger.viewer.options
		options.build_dialog()
		options.ui.dialog:popup(iup.CENTER, iup.CENTER)
		options.ui.dialog:destroy()
	end}
ledger.viewer.ui.close          = iup.stationbutton{title="Close", action = function(self)
		if (ledger.viewer.updated) then 
			ledger.save(ledger.list)
			ledger.viewer.updated = false
		end
		HideDialog(ledger.viewer.ui.dialog)
	end}

function ledger.viewer.build_dialog()

	local function subdlg(ctrl)
		return iup.dialog{
			ctrl,
			border="NO",
			menubox="NO",
			resize="NO",
			bgcolor="0 0 0 0 *",
		}
	end

	local matrix = 	iup.stationsubframe{
		iup.vbox{
			iup.vbox{ledger.viewer.ui.matrix,},
			iup.hbox{
				ledger.viewer.ui.searchbox,
				ledger.viewer.ui.search,
				ledger.viewer.ui.clear,
				gap = 5,
				alignment = 'ACENTER',
			},
			gap = 10,
		},
	}

	local list = iup.stationsubsublist{{}, control = 'YES', expand = 'YES', size = 175}
	local filter_controls = iup.vbox{
		ledger.viewer.filter:New('guild'),
		ledger.viewer.filter:New('player'),
		ledger.viewer.filter:New('station'),
		ledger.viewer.filter:New('ship'),
		ledger.viewer.filter:New('type'),
		ledger.viewer.filter:New('date'),
	}
	iup.Append(list, subdlg(filter_controls))
	list[1] = 1

	local filters = iup.stationsubframe{
		iup.vbox{
			label('Filters'), 
			list,
			fill(false, 5),
			iup.hbox{ fill(), ledger.viewer.ui.filter_button, fill(), ledger.viewer.ui.reset, fill(),}, 
			fill(false, 20),
			iup.hbox{
				label('Total:'),
				ledger.viewer.ui.total,
				gap = 5,
			},
		},
	}
		
	ledger.viewer.ui.main = iup.pdarootframe{
		iup.pdarootframebg{
			iup.vbox{
				iup.hbox{label('Transaction Ledger [v'..ledger.version..']'), iup.fill{},},
				iup.hbox{
					matrix,
					filters,
					gap=5, 
					alignment="ATOP",
				},
				iup.hbox{
					iup.fill{},
					ledger.viewer.ui.options,
					ledger.viewer.ui.close,
					gap = 10,
				},
				gap=5,
			},
		},
		expand="NO",
	}

	ledger.viewer.ui.dialog = iup.dialog{
		iup.vbox{
			iup.fill{},
				iup.hbox{
					iup.fill{},
					ledger.viewer.ui.main,
					iup.fill{},
				},
			iup.fill{},
		},
		defaultenter = ledger.viewer.ui.search,
		defaultesc = ledger.viewer.ui.close,
		bgcolor = "0 0 0 128 *",
		fullscreen = "YES",
		border = "NO",
		resize = "NO",
		menubox = "YES",
		topmost = "YES",
	}

	function ledger.viewer.ui.dialog:Show()
		-- test.print(sortlist(ledger.list, ledger.viewer.sort_col, ledger.viewer.sort_dir))
		sortlist(ledger.list, ledger.viewer.sort_col, ledger.viewer.sort_dir)
		ledger.viewer.ui.matrix:update()
		ShowDialog(ledger.viewer.ui.dialog)
	end
	ledger.viewer.ui.dialog:map()
end

dofile('ui_edit.lua')
dofile('ui_options.lua')
